declare type Func<Args extends any[]> = (...args: Args) => any;
declare type AnyFunc = Func<any[]>;
declare type Listen<K extends string, Args extends any[]> = (name: K, listener: (...args: Args) => any, ...rest: any[]) => any;
declare type AnyListen = Listen<any, any[]>;
declare type Listenable = {
    on: AnyListen;
    off: AnyListen;
} | {
    addListener: AnyListen;
    removeListener: AnyListen;
} | {
    addEventListener: AnyListen;
    removeEventListener: AnyListen;
};
declare type EventName<O extends Listenable> = O extends Window ? keyof WindowEventMap : O extends Document ? keyof DocumentEventMap : O extends HTMLElement ? keyof HTMLElementEventMap : O extends WebSocket ? keyof WebSocketEventMap : O extends XMLHttpRequest ? keyof XMLHttpRequestEventMap : O extends {
    on: AnyFunc;
} ? Parameters<O['on']>[0] : O extends {
    addListener: AnyFunc;
} ? Parameters<O['addListener']>[0] : O extends {
    addEventListener: AnyFunc;
} ? Parameters<O['addEventListener']>[0] : string;
declare type Listener<O extends Listenable, K extends string> = O extends Window ? K extends keyof WindowEventMap ? Func<[WindowEventMap[K]]> : never : O extends Document ? K extends keyof DocumentEventMap ? Func<[DocumentEventMap[K]]> : never : O extends HTMLElement ? K extends keyof HTMLElementEventMap ? Func<[HTMLElementEventMap[K]]> : never : O extends WebSocket ? K extends keyof WebSocketEventMap ? Func<[WebSocketEventMap[K]]> : never : O extends XMLHttpRequest ? K extends keyof XMLHttpRequestEventMap ? Func<[XMLHttpRequestEventMap[K]]> : never : O extends {
    on: Listen<K, infer Args>;
} ? Func<Args> : O extends {
    addListener: Listen<K, infer Args>;
} ? Func<Args> : O extends {
    addEventListener: Listen<K, infer Args>;
} ? Func<Args> : AnyFunc;
export declare function add<O extends Listenable, K extends EventName<O>>(o: O, name: K, listener: Listener<O, K>, ...params: any[]): {
    (): void;
    add<O_1 extends Listenable, K_1 extends EventName<O_1>>(o: O_1, name: K_1, listener: Listener<O_1, K_1>, ...params: any[]): any;
};
export default add;
